<?php

namespace SM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SM\Service\SmPropostasImpostosService;
use SM\Traits\TResponse;

class SmPropostasImpostosController
{
    use TResponse;
    protected $propostaImpostoService;

    public function __construct(SmPropostasImpostosService $propostaImpostoService)
    {
        $this->propostaImpostoService = $propostaImpostoService;
    }

    public function getImpostoproposta()
    {
        try {
            $data = $this->propostaImpostoService->getImpostoproposta();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function getTabComposicoesProposta(Request $request) 
    {
        try {
            $parameters = $request->getParameters();
            $data = $this->propostaImpostoService->getTabComposicoesProposta($parameters);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }    

    public function saveComposicaoServicoTab(Request $request) 
    {
        try {
            $params = $request->getParameters();
            $data = $this->propostaImpostoService->saveComposicaoServicoTab($params);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }    

    public function getComposicoesProposta(Request $request) 
    {
        try {
            $parameters = $request->getParameters();
            $data = $this->propostaImpostoService->getComposicoesProposta(  $parameters);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }  

    public function saveTabImposto(DTO\Request $request)
    {
        try {
            $data = $this->propostaImpostoService->saveTabImposto($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteImposto(Request $request)
    {
        try {
            $data = $this->propostaImpostoService->deleteImposto($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }

    public function deleteImpostoComposicaoServ(Request $request)
    {
        try {
            $data = $this->propostaImpostoService->deleteImpostoComposicaoServ($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }
    
    public function salvarEdicaoImposto(Request\Row $request, Response $response){
        try{
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrorg();
            $cdimposto     = $row["CDIMPOSTO"];
            $vrprecimposto = $row["VRPRECIMPOSTO"];
            $this->impostoservice->salvarEdicaoImposto($cdimposto, $nrorg, $vrprecimposto);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));        
        }
    }
}