<?php

namespace SM\Service;

use SM\Factory\SmPropostasAlimentacaoFactory;
use Teknisa\Libs\Util\Environment;
use SM\Service\FindService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SmPropostasAlimentacaoService
{
    protected $connection;
    protected $PropostaAlimentacaoFactory;
    protected $environment;

    protected $findService;

    protected $newCode;

    public function __construct(SmPropostasAlimentacaoFactory $PropostaAlimentacaoFactory, Environment $environment, FindService $findService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->PropostaAlimentacaoFactory = $PropostaAlimentacaoFactory;
        $this->environment = $environment;
        $this->findService = $findService;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
    }

    public function getPropostasAlimentacao()
    {
        return $this->PropostaAlimentacaoFactory->getPropostasAlimentacao();
    }

    public function getEscopoFaturamentoProposta($params){
        return $this->PropostaAlimentacaoFactory->getEscopoFaturamentoProposta($params);
    }

    public function getTabServicosPropostas($nrtipoorcamento)
    {
        return $this->PropostaAlimentacaoFactory->getTabServicosPropostas($nrtipoorcamento);
    }

    public function getInvestimentosPropostaAlimentacao($params)
    {

        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );
        return $this->PropostaAlimentacaoFactory->getInvestimentosPropostaAlimentacao($params);
    }

    public function getComposicoesServicos($nrtipoorcamento, $nrcomposicao){
        $nrtipoorcamento = implode(",", $nrtipoorcamento);
        $nrcomposicao    = implode(",", array_unique($nrcomposicao));
       return $this->PropostaAlimentacaoFactory->getElementosComposicaoServico($nrtipoorcamento, $nrcomposicao);
    }

    public function getMaoDeObraQuant($params){
        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );
        return $this->PropostaAlimentacaoFactory->getMaoDeObraQuant($params);
    }

    public function getTabServicosPropostas($parametros)
    {
    
        $parametros = array(
            "NRORCAMENTOMODELO" => $parametros['NRORCAMENTOMODELO'],
            "NRTIPOORCAMENTO" => $parametros['NRTIPOORCAMENTO'],
        );
        return $this->PropostaAlimentacaoFactory->getTabServicosPropostas($parametros);
    }
    

    public function getInvestimentos($params)
    {
       
        return $this->PropostaAlimentacaoFactory->getInvestimentos($params);
    }

    public function getParNegocioProposta()
    {
       
        return $this->PropostaAlimentacaoFactory->getParNegocioProposta();
    }

    public function getDespesas($params)
    {
       
        return $this->PropostaAlimentacaoFactory->getDespesas($params);
    }

    public function getDespesasPropostaAlimentacao( $params)
    {
        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );
        return $this->PropostaAlimentacaoFactory->getDespesasPropostaAlimentacao( $params);
    }

    public function getImpostos()
    {
        return $this->PropostaAlimentacaoFactory->getImpostos();
    }

    public function getImpostosPropostaAlimentacao($params)
    {
        return $this->PropostaAlimentacaoFactory->getImpostosPropostaAlimentacao($params);
    }
    
    public function getServicoRelacao($params)
    {
        // $params = array(
        //     "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        // );
        return $this->PropostaAlimentacaoFactory->getServicoRelacao($params);
    }

    public function getInvestimentosPropostaAlimentacao($params)
    {
        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );

        return $this->PropostaAlimentacaoFactory->getInvestimentosPropostaAlimentacao($params);
    }

    public function getItensTipoElementoProposta($params)
    {

        return $this->PropostaAlimentacaoFactory->getItensTipoElementoProposta($params);
    }

    public function getComposicoesServicos()
    {
        $elementos = $this->PropostaAlimentacaoFactory->getElementosComposicaoServicoSemNrComposicao();
        $elementosValidados = $this->validarExistenciaElementoRaiz($elementos);
        $elementosAgrupados = $this->agruparElementosPorComposicao($elementosValidados);
    
        return $elementosAgrupados;
    }

    private function agruparElementosPorComposicao($elementos)
    {
        $agrupados = [];
    
        foreach ($elementos as $elemento) {
            $nrComposicao = isset($elemento["NRCOMPSERVICO"]) && empty($elemento["NRCOMPSERVICO"])
                ? $elemento["NRCOMPSERVICO"]
                : "SEM_COMPOSICAO";
    
            if (!isset($agrupados[$nrComposicao])) {
                $agrupados[$nrComposicao] = [
                    "NRCOMPSERVICO" => $nrComposicao,
                    "ELEMENTOS" => []
                ];
            }

            $agrupados[$nrComposicao]["ELEMENTOS"][] = $elemento;
        }
        return array_values($agrupados); // Retorna os grupos formatados corretamente
    }
    
        public function validarExistenciaElementoRaiz($elementos)
    {
        $arrElemenstosCriados = array();
        foreach($elementos as $key=>&$elemento){
            if(isset($elemento["IDELEMENTOPAI"]) && $elemento["IDELEMENTOPAI"] !== null && !in_array($elemento["IDELEMENTOPAI"], $arrElemenstosCriados)){
                $existe = false;
                $count  = 0;
                foreach ($elementos as $eleRaiz) {
                    if(isset($elemento["IDELEMENTO"]) && $elemento["IDELEMENTO"] !== $eleRaiz["IDELEMENTO"] && $elemento["IDELEMENTOPAI"] == $eleRaiz["IDELEMENTO"]){
                        if($count < 2){
                            $count = $count + 1;
                        } else { 
                            $existe = true;
                            break;
                        }
                    }
                }
                
                if(!$existe){
                    $elementoPai = $elemento;
                    $elementoPai["IDELEMENTO"]      = $elemento["IDELEMENTOPAI"];
                    $elementoPai["NMSERVORCARELAC"] = "";
                    $elementoPai["CDSERVORCARELAC"] = "";
                    $elementoPai["IDELEMENTOPAI"]   = null;
                    $elementos[] = $elementoPai;
                    array_push($arrElemenstosCriados, $elemento["IDELEMENTOPAI"]);
                }
            }
        }
        return $elementos;
    }    

    public function getMaodeObra($params){
        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );
        
        return $this->PropostaAlimentacaoFactory->getMaodeObra($params);
    }

   
    public function salvarPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            $nrSeqOrcamento = $this->newCode->getCodeByTableName("FCT_ORCAMENTO", $nrorg, 1, 10, true);
            if (empty($params["NRSEQORCAMENTO"]) && $params["NRSEQORCAMENTO"] == null) {
                $parametrosProposta = array(
                    "NRSEQORCAMENTO" => $nrSeqOrcamento,
                    "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                    "NRVERSAOORCAMENTO" => $params["NRVERSAOORCAMENTO"],
                    "DTREFORCAMENTO" => $params["DTREFORCAMENTO"],
                    "IDSITUACAOORCAMENTO" => $params["IDSITUACAOORCAMENTO"],
                    "VRORCAMENTO" => $params["VRORCAMENTO"],
                    "DTACEITEORCA" => $params["DTACEITEORCA"],
                    "NRPARCNEGOCIO" => $params["NRPARCNEGOCIO"],
                    "VRPERCMARKUP" => $params["VRPERCMARKUP"],
                    "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                    "DTVALIDADE" => $params["DTVALIDADE"],
                    "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "IDATIVO" => 'S',
                );      
                
                $this->PropostaAlimentacaoFactory->inserirProposta($parametrosProposta);
            } else {
                $parametrosProposta = array(
                   "NRSEQORCAMENTO" => isset($params["NRSEQORCAMENTO"]),
                    "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                    "NRVERSAOORCAMENTO" => $params["NRVERSAOORCAMENTO"],
                    "DTREFORCAMENTO" => $params["DTREFORCAMENTO"],
                    "IDSITUACAOORCAMENTO" => $params["IDSITUACAOORCAMENTO"],
                    "VRORCAMENTO" => $params["VRORCAMENTO"],
                    "DTACEITEORCA" => $params["DTACEITEORCA"],
                    "NRPARCNEGOCIO" => $params["NRPARCNEGOCIO"],
                    "VRPERCMARKUP" => isset($params["VRPERCMARKUP"]),
                    "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                    "DTVALIDADE" => $params["DTVALIDADE"],
                    "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                    "NRORGULTATU"   => $nrorg,
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "CDOPERINCLUSAO" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarProposta($parametrosProposta);
            }
    
            // if (empty($params["NRFORMPRECOSERVICO"]) && $params["NRFORMPRECOSERVICO"] == null) {
            //     $nrCalcFormPrecServ = $this->newCode->getCodeByTableName("FCT_CALCFORMPRECSERV", $nrorg, 1, 10, true);
                
            //     $parametrosCalcFormPrecServ = array(
            //         "NRCALCFORMPRECSERV" => $nrCalcFormPrecServ,
            //         "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //          "NRORG" => $nrorg,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->inserirCalcFormPrecServ($parametrosCalcFormPrecServ);
            // } else {
            //     $parametrosCalcFormPrecServ = array(
            //         "NRCALCFORMPRECSERV" => $params["NRCALCFORMPRECSERV"],
            //         "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->atualizarCalcFormPrecServ($parametrosCalcFormPrecServ);
            // }
    
            // if (isset($params["NRRELACSERVORCA"]) && empty($params["NRRELACSERVORCA"]) && $params["NRRELACSERVORCA"] == null) {
            //     $parametrosServicoOrcamento = array(
            //         "NRRELACSERVORCA" => $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true),
            //         "QTSEGUNDA" => $params["QTSEGUNDA"],
            //         "QTTERCA" => $params["QTTERCA"],
            //         "QTQUARTA" => $params["QTQUARTA"],
            //         "QTQUINTA" => $params["QTQUINTA"],
            //         "QTSEXTA" => $params["QTSEXTA"],
            //         "QTSABADO" => $params["QTSABADO"],
            //         "QTDOMINGO" => $params["QTDOMINGO"],
            //         "NRORG" => $nrorg,
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->inserirRelacServOrcamento($parametrosServicoOrcamento);
            // } else {
            //     $parametrosServicoOrcamento = array(
            //         "NRRELACSERVORCA" => isset($params["NRRELACSERVORCA"]),
            //         "QTSEGUNDA" => $params["QTSEGUNDA"],
            //         "QTTERCA" => $params["QTTERCA"],
            //         "QTQUARTA" => $params["QTQUARTA"],
            //         "QTQUINTA" => $params["QTQUINTA"],
            //         "QTSEXTA" => $params["QTSEXTA"],
            //         "QTSABADO" => $params["QTSABADO"],
            //         "QTDOMINGO" => $params["QTDOMINGO"],
            //         "NRORG" => $nrorg,
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->atualizarRelacServOrcamento($parametrosServicoOrcamento);
            // }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarServicosPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
        
            if (empty($params["NRRELACSERVORCA"]) && $params["NRRELACSERVORCA"] == null) {
                $parametrosServicoOrcamento = array(
                    "NRRELACSERVORCA" => $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true),
                    "NRSEQORCAMENTO" => $params["NRSEQORCAMENTO"],
                    "QTSEGUNDA" => $params["QTSEGUNDA"],
                    "QTTERCA" => $params["QTTERCA"],
                    "QTQUARTA" => $params["QTQUARTA"],
                    "QTQUINTA" => $params["QTQUINTA"],
                    "QTSEXTA" => $params["QTSEXTA"],
                    "QTSABADO" => $params["QTSABADO"],
                    "QTDOMINGO" => $params["QTDOMINGO"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->inserirRelacServOrcamento($parametrosServicoOrcamento);
            } else {
                $parametrosServicoOrcamento = array(
                    "NRRELACSERVORCA" => isset($params["NRRELACSERVORCA"]),
                    "QTSEGUNDA" => $params["QTSEGUNDA"],
                    "QTTERCA" => $params["QTTERCA"],
                    "QTQUARTA" => $params["QTQUARTA"],
                    "QTQUINTA" => $params["QTQUINTA"],
                    "QTSEXTA" => $params["QTSEXTA"],
                    "QTSABADO" => $params["QTSABADO"],
                    "QTDOMINGO" => $params["QTDOMINGO"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarRelacServOrcamento($parametrosServicoOrcamento);
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveTabMaodeObra($params){
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
           if (empty($params["NRCALCFORMPRECSERV"]) && $params["NRCALCFORMPRECSERV"] == null) {
                $nrCalcFormPrecServ = $this->newCode->getCodeByTableName("FCT_CALCFORMPRECSERV", $nrorg, 1, 10, true);
                
                $parametrosCalcFormPrecServ = array(
                    "NRCALCFORMPRECSERV" => $nrCalcFormPrecServ,
                    "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                     "NRORG" => $nrorg,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->inserirCalcFormPrecServ($parametrosCalcFormPrecServ);
            } else {
                $parametrosCalcFormPrecServ = array(
                    "NRCALCFORMPRECSERV" => $params["NRCALCFORMPRECSERV"],
                    "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarCalcFormPrecServ($parametrosCalcFormPrecServ);
            }
              
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    
    }



// public function saveInvestimentoPropostaAlimentacao($params)
// {
//     $this->connection->beginTransaction();

//     try {
//         $nrorg = $this->environment->getNrorg();
//         $operador = $this->environment->getCdOperador();

//         foreach ($params["DRG"] as $row) {
//             if ($row["NRINVESTIMENTO"] != 0) {
                
//                 // Atualiza a relação investimento-proposta
//                 $paramInvestProposta = [
//                     "NRINVESTPROP" => $row["NRINVESTPROP"],
//                     "VRUNITINVESTORCA" => $row["VRINVESTIMENTO"],
//                     "VRTOTINVETORCA" => $row["VRINVESTIMENTO"] * $row["QTINVESTORCA"],
//                     "QTINVESTORCA" => $row["QTINVESTORCA"]
//                 ];
//                 $this->PropostaAlimentacaoFactory->atualizarInvestimento($paramInvestProposta);
//             } else {
//                 // Cria novo investimento
//                 // Insere a relação investimento-proposta
//                 $nrInvestProp = $this->newCode->getCodeByTableName("FCT_RELACINVESTPROPOSTA", $nrorg, 1, 10, true);
//                 $paramInvestProposta = [
//                         "NRINVESTPROP" =>  $nrInvestProp,
//                         "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
//                         "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
//                         "DSINVESTIMENTO" =>$row["DSINVESTIMENTO"],
//                         "VRUNITINVESTORCA" => $row["VRUNITINVESTORCA"],
//                         "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"],
//                         "NRORG" => $nrorg,
//                         "NRORGINCLUSAO" => $nrorg,
//                         "CDOPERINCLUSAO" => $operador,
//                         "IDATIVO" => $row["IDATIVO"]
//                 ];
//                 $this->PropostaAlimentacaoFactory->inserirInvestimento($paramInvestProposta);
//             }
//         }

//         $this->connection->commit();
//     } catch (\Exception $e) {
//         $this->connection->rollback();
//         throw $e;
//     }
// }

    public function saveDespesasPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
    
            foreach ($params["DRG"] as $row) {
                if ($row["CDDESPESA"] != 0) {
                    $parametros = array(
                        "CDDESPESA"       => $row["CDDESPESA"],
                        "DSDESPESA"       => $row["DSDESPESA"],
                        "VRDESPESA"       => $row["VRDESPESA"],
                        "CDPERIODO"       => $row["CDPERIODO"],
                        "IDATIVO"         => $row["IDATIVO"],
                        "NRORGULTATU"     => $nrorg,
                        "CDOPERULTATU"    => $operador
                    );
    
                    $this->PropostaAlimentacaoFactory->atualizarDespesa($parametros);
                } else {
                    $cdDespesa = $this->newCode->getCodeByTableName("FCT_DESPESA", $nrorg, 1, 10, true);
    
                    $parametros = array(
                        "CDDESPESA"       => $cdDespesa,
                        "DSDESPESA"       => $row["DSDESPESA"],
                        "VRDESPESA"       => $row["VRDESPESA"],
                        "CDPERIODO"       => $row["CDPERIODO"],
                        "NRORG"           => $nrorg,
                        "NRORGINCLUSAO"   => $nrorg,
                        "CDOPERINCLUSAO"  => $operador,
                        "IDATIVO"         => $row["IDATIVO"]
                    );
    
                    $this->PropostaAlimentacaoFactory->inserirDespesa($parametros);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }  

    public function saveImpostoPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            foreach ($params["DRG"] as $row) {
                if ($row["NRIMPOSTOSERV"] != 0) {
                    $parametros = array(
                        "CDIMPOSTO" => $row["CDIMPOSTO"],
                        "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                        "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => "S",
                        "NRIMPOSTOSERV" => $row["NRIMPOSTOSERV"],
                    );
                    
                    $this->PropostaAlimentacaoFactory->updateTabImposto($parametros);
                } else {
                    $NRIMPOSTOSERV = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOSERV", $nrorg, 1, 10, true);
                    
                    $parametros = array(
                    "NRIMPOSTOSERV" =>  $NRIMPOSTOSERV,
                    "CDIMPOSTO" =>     $row["CDIMPOSTO"],
                    "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                    "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                    "NRORG" => $nrorg,
                    "IDATIVO" => "S",
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                    );
                    
                    $this->PropostaAlimentacaoFactory->saveTabImposto($parametros);
                }
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarServicosPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
    
            foreach ($params["DRG"] as $row) {
                if (empty($row["NRRELACSERVORCA"])) {
                    $nrRelacServOrca = $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true);
                    $parametrosServicoOrcamento = array(
                        "NRRELACSERVORCA" => $nrRelacServOrca,
                        "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                        "IDESCOLHIDO" => $row["IDESCOLHIDO"] ?? 'S',
                        "IDTIPOELEMENTO" => $row["IDTIPOELEMENTO"] ?? 'P',
                        "QTSEGUNDA" => $row["QTSEGUNDA"] ?? null,
                        "QTTERCA" => $row["QTTERCA"]?? null,
                        "QTQUARTA" => $row["QTQUARTA"] ?? null,
                        "QTQUINTA" => $row["QTQUINTA"]?? null,
                        "QTSEXTA" => $row["QTSEXTA"]?? null,
                        "QTSABADO" => $row["QTSABADO"]?? null,
                        "QTDOMINGO" => $row["QTDOMINGO"]?? null,
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => 'S',
                    );
                    $this->PropostaAlimentacaoFactory->inserirRelacServOrcamento($parametrosServicoOrcamento);
                } else {
                    $parametrosServicoOrcamento = array(
                        "NRRELACSERVORCA" => $row["NRRELACSERVORCA"],
                        "QTSEGUNDA" => $row["QTSEGUNDA"],
                        "QTTERCA" => $row["QTTERCA"],
                        "QTQUARTA" => $row["QTQUARTA"],
                        "QTQUINTA" => $row["QTQUINTA"],
                        "QTSEXTA" => $row["QTSEXTA"],
                        "QTSABADO" => $row["QTSABADO"],
                        "QTDOMINGO" => $row["QTDOMINGO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => 'S',
                    );
                    $this->PropostaAlimentacaoFactory->atualizarRelacServOrcamento($parametrosServicoOrcamento);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function saveTabMaodeObra($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            var_dump($params);
            foreach ($params["DRG"] as $row) {
                if (empty($row["NRRELACFORMACAOPROP"])) {
                    $nrRelacFormacaoProp = $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRRELACFORMACAOPROP" => $nrRelacFormacaoProp,
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                        // "NRITEMORCAMENTO" => isset($row["NRITEMORCAMENTO"]) ?? null,
                        "NRORG" => $nrorg,
                        "IDATIVO" => 'S',
                        "DTINCLUSAO" => date("Y-m-d H:i:s"),
                        "DTULTATU" => date("Y-m-d H:i:s"),
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "QTFORMACAOPROP" => $row["QTFORMACAOPROP"] ?? null,
                        "CDSERVORCARELAC" => $row["CDSERVORCARELAC"] ?? null,
                        );
    
                        $this->PropostaAlimentacaoFactory->inserirRelacFormacaoProp($parametros);
                        var_dump($params);
                } else {
                    $nrRelacFormacaoProp = $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRRELACFORMACAOPROP" => $row["NRRELACFORMACAOPROP"],
                        "NRFORMPRECOSERVICO" =>  $row["NRFORMPRECOSERVICO"],
                        "NRSEQORCAMENTO" =>  $row["NRSEQORCAMENTO"],
                        // "NRITEMORCAMENTO" =>  $row["NRITEMORCAMENTO"],
                        "NRORG" => $nrorg,
                        "IDATIVO" => 'S',
                        "DTINCLUSAO" => date("Y-m-d H:i:s"),
                        "DTULTATU" => date("Y-m-d H:i:s"),
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "QTFORMACAOPROP" =>  $row["QTFORMACAOPROP"] ?? null,
                        "CDSERVORCARELAC" =>  $row["CDSERVORCARELAC"] ?? null,
                    );
                    $this->PropostaAlimentacaoFactory->atualizarRelacFormacaoProp($parametros);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }   

    public function saveInvestimentoPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
    
            foreach ($params["DRG"] as $row) {
                if ($row["NRINVESTPROP"] != 0) {
                    // Atualizar investimento existente
                    $paramInvestProposta = [
                        "NRINVESTPROP" => $row["NRINVESTPROP"],
                        "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "VRUNITINVESTORCA" => $row["VRUNITINVESTORCA"],
                        "VRTOTINVETORCA" => $row["VRTOTINVETORCA"],
                        "QTINVESTORCA" => $row["QTINVESTORCA"],
                        "DTULTATU" => date('Y-m-d H:i:s'),
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"],
                        "VRPARCELA" => $row["VRPARCELA"],
                        "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"]
                    ];
                    $this->PropostaAlimentacaoFactory->atualizarInvestimento($paramInvestProposta);
                } else {
                    // Inserir novo investimento
                    $nrInvestProp = $this->newCode->getCodeByTableName("FCT_RELACINVESTPROPOSTA", $nrorg, 1, 10, true);
                    $paramInvestProposta = [
                        "NRINVESTPROP" => $nrInvestProp,
                        "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "VRUNITINVESTORCA" => $row["VRUNITINVESTORCA"],
                        "VRTOTINVETORCA" => $row["VRTOTINVETORCA"],
                        "QTINVESTORCA" => $row["QTINVESTORCA"],
                        "DTINCLUSAO" => date('Y-m-d H:i:s'),
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "IDATIVO" => $row["IDATIVO"],
                        "VRPARCELA" => $row["VRPARCELA"],
                        "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"]
                    ];
                    $this->PropostaAlimentacaoFactory->inserirInvestimento($paramInvestProposta);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveDespesasPropostaAlimentacao($params) 
{
    $this->connection->beginTransaction();

    try {
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();

        foreach ($params["DRG"] as $row) {
            if ($row["NRDESPPROP"] != 0) { // Atualiza se já existe
                $parametros = [
                    "NRDESPPROP"         => $row["NRDESPPROP"],
                    "NRSEQORCAMENTO"     => $row["NRSEQORCAMENTO"],
                    "CDDESPESA"          => $row["CDDESPESA"],
                    "VRDESPESAORCA"      => $row["VRDESPESAORCA"],
                    "CDPERIODODESPPROP"  => $row["CDPERIODODESPPROP"],
                    "VRUNITDESPESA"      => $row["VRUNITDESPESA"] ?? null,
                    "IDATIVO"            => 'S',
                    "NRORGULTATU"        => $nrorg,
                    "CDOPERULTATU"       => $operador
                ];

                $this->PropostaAlimentacaoFactory->atualizarDespesa($parametros);
            } else {
                $nrDespProp = $this->newCode->getCodeByTableName("FCT_RELACDESPPROPOSTA", $nrorg, 1, 10, true);

                $parametros = [
                    "NRDESPPROP"         => $nrDespProp,
                    "NRSEQORCAMENTO"     => $row["NRSEQORCAMENTO"],
                    "CDDESPESA"          => $row["CDDESPESA"],
                    "VRDESPESAORCA"      => $row["VRDESPESAORCA"],
                    "CDPERIODODESPPROP"  => $row["CDPERIODODESPPROP"],
                    "VRUNITDESPESA"      => $row["VRUNITDESPESA"] ?? null,
                    "NRORG"              => $nrorg,
                    "NRORGINCLUSAO"      => $nrorg,
                    "CDOPERINCLUSAO"     => $operador,
                    "IDATIVO"            => 'S',
                ];

                $this->PropostaAlimentacaoFactory->inserirDespesa($parametros);
            }
        }

        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e;
    }
}

public function saveImpostoPropostaAlimentacao($params)
{
    $this->connection->beginTransaction();

    try {
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();

        foreach ($params["DRG"] as $row) {
            if ($row["NRIMPOSTOORCA"] != 0) { // Atualiza se já existe
                $parametros = [
                    "NRIMPOSTOORCA"  => $row["NRIMPOSTOORCA"],
                    "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                    "CDIMPOSTO"      => $row["CDIMPOSTO"],
                    "VRPERCIMPOSTO"  => $row["VRPERCIMPOSTO"],
                    "NRORGULTATU"    => $nrorg,
                    "CDOPERULTATU"   => $operador,
                    "IDATIVO"        => "S"
                ];

                $this->PropostaAlimentacaoFactory->updateTabImposto($parametros);
            } else {
                $nrImpostoOrca = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOORCA", $nrorg, 1, 10, true);

                $parametros = [
                    "NRIMPOSTOORCA"  => $nrImpostoOrca,
                    "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
                    "CDIMPOSTO"      => $row["CDIMPOSTO"],
                    "VRPERCIMPOSTO"  => $row["VRPERCIMPOSTO"],
                    "NRORG"          => $nrorg,
                    "IDATIVO"        => "S",
                    "NRORGINCLUSAO"  => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                ];
                $this->PropostaAlimentacaoFactory->saveTabImposto($parametros);
            }
        }

        $this->connection->commit();
    } catch (\Exception $e) {
        $this->connection->rollback();
        throw $e;
    }
}

    public function calcularValorOrcamento(array $params){
        $taxas        = $this->PropostaAlimentacaoFactory->getVrTaxasProposta($params);
        $mObras       = $this->PropostaAlimentacaoFactory->getVrMObraProposta($params);
        $invetimentos = $this->PropostaAlimentacaoFactory->getVrInvestProposta($params);
        $impostos     = $this->PropostaAlimentacaoFactory->getVrImpostosProposta($params);
        $despesas     = $this->PropostaAlimentacaoFactory->getVrDespesasProposta($params);
        $servicos     = $this->PropostaAlimentacaoFactory->getVrServicosProposta($params);
        $markups      = $this->PropostaAlimentacaoFactory->getVrMarkup($params);
        $markupAplic  = $markups["VRPERCMARKUPAPLICADO"];
        $markupIdeal  = $markups["VRPERCMARKUP"];
        
        $totalOrc   = $mObras + $invetimentos + $despesas + $servicos;
        $totalStaxa = $totalOrc;
        
        $totalOrc *= (1 + ($taxas/100));
        $totalCtaxa = $totalOrc;
        
        $totalOrcIdeal = $totalOrc;
        $totalOrcIdeal *= (1 + ($markupIdeal/100));
        $totalOrc      *= (1 + ($markupAplic/100));
        
        $datasetFin = array(
            "TOTALBRUTO"    => $totalOrc,
            "IMPOSTO"       => ($totalCtaxa * (1 + ($markupAplic/100))) * ($impostos/100),
            "TOTALLIQ"      => $totalOrc - ($totalOrc * ($impostos/100)),
            "CALIMENTAR"    => $servicos,
            "INVESTIMENTOS" => $invetimentos,
            "DESPESAS"      => $despesas,
            "TAXAS"         => $totalStaxa * ($taxas/100),
            "MARKUP"        => $totalCtaxa * ($markupAplic/100),
            "MOBRA"         => $mObras,
            "TOTALCTAXA"    => $totalCtaxa,
            "TOTALSTAXA"    => $totalStaxa,
            "TOTALIDEAL"    => $totalOrcIdeal,
            "IMPOSTOIDEAL"  => $totalOrcIdeal * ($impostos/100),
            "TOTALLIQIDEAL" => $totalOrcIdeal - ($totalOrcIdeal * ($impostos/100)),
            "MARKUPIDEAL"   => $totalCtaxa * ($markupIdeal/100)
        );
        
        return $datasetFin;
    }
    /**
     * Calcula o valor total de cada serviço do orcamento
    **/
    public function calculaValoresServicos($totalOrca, $totalIdeal, $params){
    	$arrServ = array();
    	$dadosServ = $this->PropostaAlimentacaoFactory->getInfoServicosProp($params);
    	foreach($dadosServ as $dadoServ){
    	    $qttotal = $dadoServ["QTTOTALORCA"] <= 0 ? 1 : $dadoServ["QTTOTALORCA"];
    		$servico = array(
    			"NRCOMPSERVICO"       => $dadoServ["NRCOMPSERVICO"],
                "CDSERVORCA"          => explode("_", $dadoServ["CDSERVORCA"]),
                "NMELEMENTO"          => $dadoServ["NMELEMENTO"],
                "VRCUSTO"             => $dadoServ["VRCUSTOSERV"],
                "VRPERCSERVICO"       => $dadoServ["VRPERCSERVICO"],
                "VRPRECOIDEAL"        => $dadoServ["VRCUSTOIDEAL"],
                "VRUNITITEMORCA"      => $dadoServ["VRCUSTOINFORMADO"],
                "VRUNITITEMORCA_GRID" => $dadoServ["VRCUSTOINFORMADO"],
                "VRTOTALITEMORCA"     => $dadoServ["VRTOTSERV"],
                "VRTOTIDEALSERV"      => ($totalIdeal/$qttotal) * $dadoServ["QTTOTALSERV"],
                "QTITEMORCAMENTO"     => $dadoServ["QTTOTALSERV"],
                "NRTIPOORCAMENTO"     => explode("_", $dadoServ["NRTIPOORCAMENTO"])
    		);
    
    		array_push($arrServ, $servico);
    	}
    	return $arrServ;
    }

    public function getInfoResultado($nrseqorcamento){
        $params = array(
            "NRSEQORCAMENTO" => $nrseqorcamento,
        );
        
        $financeiro = $this->calcularValorOrcamento($params);
        $servicos   = $this->calculaValoresServicos($financeiro["TOTALBRUTO"], $financeiro["TOTALIDEAL"], $params);
        $resultado  = array(
            "SERVICOS"   => $servicos,
            "FINANCEIRO" => $financeiro
        );
        
        return $resultado;
    }

    public function deletePropostaAlimenticia($parametros)
    {
        $this->connection->beginTransaction();
    
        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
    
            $this->PropostaAlimentacaoFactory->deletePropostaAlimenticia($retorno);
            $this->PropostaAlimentacaoFactory->deleteImpostosRelac($retorno);
            $this->PropostaAlimentacaoFactory->deleteDespesasRelac($retorno);
            $this->PropostaAlimentacaoFactory->deleteInvestimentosRelac($retorno);
            $this->PropostaAlimentacaoFactory->deleteMaodeObra($retorno);
            $this->PropostaAlimentacaoFactory->deleteServicosRelac($retorno);
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}