<?php

namespace SM\Service;

use SM\Factory\SmPropostasDespesasFactory;
use Teknisa\Libs\Util\Environment;
use SM\Service\FindService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SmPropostasDespesasService
{
    protected $connection;
    protected $despesasFactory;
    protected $environment;

    protected $findService;

    protected $newCode;

    public function __construct(SmPropostasDespesasFactory $despesasFactory, Environment $environment, FindService $findService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->despesasFactory = $despesasFactory;
        $this->environment = $environment;
        $this->findService = $findService;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
    }

    public function getDespesasproposta()
    {
        return $this->despesasFactory->getDespesasproposta();
    }

    public function getTabFormacaoPreco($params)
    {
        $params = array(
            "CDDESPESA" => $params['CDDESPESA']
        );
        return $this->despesasFactory->getTabFormacaoPreco($params);
    }

    public function getComposicaoDespesas($params)
    {
        $params = array(
            "CDDESPESA" => $params['CDDESPESA']
        );
        return $this->despesasFactory->getComposicaoDespesas($params);
    }
    public function saveDespesas($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["CDDESPESA"] != 0) {
                    $parametros = array(
                        "CDDESPESA" => $row["CDDESPESA"],
                        "DSDESPESA" => $row["DSDESPESA"],
                        "VRDESPESA" => $row["VRDESPESA"],
                        "CDPERIODO" => $row["CDPERIODO"],
                        "IDATIVO" => $row["IDATIVO"],
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador
                    );

                    $this->despesasFactory->atualizarDespesa($parametros);
                } else {
                    $cdDespesa = $this->newCode->getCodeByTableName("FCT_DESPESA", $nrorg, 1, 10, true);

                    $parametros = array(
                        "CDDESPESA" => $cdDespesa,
                        "DSDESPESA" => $row["DSDESPESA"],
                        "VRDESPESA" => $row["VRDESPESA"],
                        "CDPERIODO" => $row["CDPERIODO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->despesasFactory->inserirDespesa($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveComposicaoServicos($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            foreach ($params["DRG"] as $row) {
                if ($row["CDRELACSERVICODESP"] != 0) {
                    $parametros = array(
                        "CDRELACSERVICODESP" => $row["CDRELACSERVICODESP"],
                        "CDDESPESA" => $row["CDDESPESA"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => $row["NRCOMPSERVICOPAI"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->despesasFactory->atualizarRelacServInvest($parametros);
                } else {
                    $CDRELACSERVICODESP = $this->newCode->getCodeByTableName("FCT_RELACSERVICODESPESA", $nrorg, 1, 10, true);
                    $parametros = array(
                        "CDRELACSERVICODESP" => $CDRELACSERVICODESP,
                        "CDDESPESA" => $row["CDDESPESA"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => isset($row["NRCOMPSERVICOPAI"]),
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "IDATIVO" => $row["IDATIVO"],
                        "CDOPERULTATU" => $operador
                    );

                    $this->despesasFactory->inserirRelacServInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveRelacFormDesp($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["CDRELACFORMDESP"] != 0) {
                    $parametros = array(
                        "CDRELACFORMDESP" => $row["CDRELACFORMDESP"],
                        "CDDESPESA" => $row["CDDESPESA"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->despesasFactory->atualizarRelacFormInvest($parametros);
                } else {
                    $nrRelacFormInvest = $this->newCode->getCodeByTableName("FCT_RELACFORMACAODESPESA", $nrorg, 1, 10, true);

                    $parametros = array(
                        "CDRELACFORMDESP" => $nrRelacFormInvest,
                        "CDDESPESA" => $row["CDDESPESA"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->despesasFactory->inserirRelacFormInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteDespesas($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->despesasFactory->deleteDespesas($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteRelacCompDespesas($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->despesasFactory->deleteRelacCompDespesas($retorno);

            $this->connection->commit();
            return 'ok';
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteFormacaoPreco($parametros)
    {
        $this->connection->beginTransaction();
        try {

            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->despesasFactory->deleteRelacFormDesp($retorno);

            $this->connection->commit();
            return 'ok';
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

}