<?php

namespace SM\Service;

use SM\Factory\SmPropostasInvestimentosFactory;
use Teknisa\Libs\Util\Environment;
use SM\Service\FindService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SmPropostasInvestimentosService
{
    protected $connection;
    protected $investimentoFactory;
    protected $environment;

    protected $findService;

    protected $newCode;

    public function __construct(SmPropostasInvestimentosFactory $investimentoFactory, Environment $environment, FindService $findService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->investimentoFactory = $investimentoFactory;
        $this->environment = $environment;
        $this->findService = $findService;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
    }

    public function getInvestimentoproposta()
    {
        return $this->investimentoFactory->getInvestimentoproposta();
    }

    public function getComposicaoInvestimento($params)
    {

        $params = array(
            "NRINVESTIMENTO" => $params['NRINVESTIMENTO']
        );
        return $this->investimentoFactory->getComposicaoInvestimento($params);
    }

    public function getTabFormacaoPreco($params)
    {
        $params = array(
            "NRINVESTIMENTO" => $params['NRINVESTIMENTO']
        );
        return $this->investimentoFactory->getTabFormacaoPreco($params);
    }

    public function getFormacaoPreco()
    {
        return $this->investimentoFactory->getFormacaoPreco();
    }

    public function saveInvestimentos($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRINVESTIMENTO"] != 0) {
                    $parametros = array(
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "DSINVESTIMENTO" => $row["DSINVESTIMENTO"],
                        "CDCATINVEST" => $row["CDCATINVEST"],
                        "VRINVESTIMENTO" => $row["VRINVESTIMENTO"],
                        "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"],
                        "IDATIVO" => $row["IDATIVO"],
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador
                    );

                    $this->investimentoFactory->atualizarInvestimento($parametros);
                } else {
                    $nrInvestimento = $this->newCode->getCodeByTableName("FCT_INVESTIMENTO", $nrorg, 1, 10, true);

                    $parametros = array(
                        "NRINVESTIMENTO" => $nrInvestimento,
                        "DSINVESTIMENTO" => $row["DSINVESTIMENTO"],
                        "CDCATINVEST" => $row["CDCATINVEST"],
                        "VRINVESTIMENTO" => $row["VRINVESTIMENTO"],
                        "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->inserirInvestimento($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveComposicaoServicos($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            foreach ($params["DRG"] as $row) {
                if ($row["NRRELACSERVINVEST"] != 0) {
                    $parametros = array(
                        "NRRELACSERVINVEST" => $row["NRRELACSERVINVEST"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => $row["NRCOMPSERVICOPAI"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->atualizarRelacServInvest($parametros);
                } else {
                    $nrRelacServInvest = $this->newCode->getCodeByTableName("FCT_RELACSERVINVESTIMENTO", $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRRELACSERVINVEST" => $nrRelacServInvest,
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => isset($row["NRCOMPSERVICOPAI"]),
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "IDATIVO" => $row["IDATIVO"],
                        "CDOPERULTATU" => $operador
                    );

                    $this->investimentoFactory->inserirRelacServInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveRelacFormInvest($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRRELACFORMINVEST"] != 0) {
                    $parametros = array(
                        "NRRELACFORMINVEST" => $row["NRRELACFORMINVEST"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->atualizarRelacFormInvest($parametros);
                } else {
                    $nrRelacFormInvest = $this->newCode->getCodeByTableName("FCT_RELACFORMINVESTIMENTO", $nrorg, 1, 10, true);

                    $parametros = array(
                        "NRRELACFORMINVEST" => $nrRelacFormInvest,
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->inserirRelacFormInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteInvest($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);

            $this->investimentoFactory->deleteFormPreco($retorno);
            $this->investimentoFactory->deleteComposi($retorno);

            $this->investimentoFactory->deleteInvest($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    public function deleteComposi($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->investimentoFactory->deleteComposi($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    public function deleteFormPreco($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->investimentoFactory->deleteFormPreco($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}