import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    DialogService, LoadingService, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class SmPropostasTaxasController {
    public tekgridVisivel: boolean = true;

    public currentRow: any = {};

    public viewTab: boolean = false;

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ CDTAXA: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSTAXA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public saveTabTaxa({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveTaxas', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public showDialogWithButtonsGrid({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasTaxasController.deleteTaxas');
    }

    public deleteTaxas() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-taxas');
        LoadingService.show();
        Http.post('/deleteTaxas', {
            NRS: grid.selectedRows.map((element: any) => element.CDTAXA),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
